@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import re
-from functools import reduce, wraps
+from functools import reduce
 
 import aiger
 import attr
@@ -13,11 +13,9 @@ from aiger_bv import common
 from aiger_bv.bundle import Bundle, BundleMap
 
 
-def support_inheritance(func):
-    @wraps(func)
-    def wrapped(*args, **kwargs):
-        return func(*args, **kwargs)
-    return wrapped
+IO_CONTRACT_VIOLATED = \
+    """Undelryiung AIG's interace incompatible with""" \
+    """AIGBV word-level interface."""
 
 
 @attr.s(frozen=True, slots=True, eq=False, auto_attribs=True)
@@ -30,6 +28,12 @@ class AIGBV:
     simulate = aiger.AIG.simulate
     simulator = aiger.AIG.simulator
 
+    def __attrs_post_init__(self):
+        # Validates io interface.
+        assert self.aig.inputs == self.imap.flat_keys, IO_CONTRACT_VIOLATED
+        assert self.aig.outputs == self.omap.flat_keys, IO_CONTRACT_VIOLATED
+        assert self.aig.latches == self.lmap.flat_keys, IO_CONTRACT_VIOLATED
+
     @property
     def aigbv(self):
         return self
@@ -58,11 +62,9 @@ class AIGBV:
         )
         return self.omap.unblast(out2val), self.lmap.unblast(latch2val)
 
-    @support_inheritance
     def __lshift__(self, other):
         return other >> self
 
-    @support_inheritance
     def __rshift__(self, other):
         interface = self.outputs & other.inputs
         assert not self.latches & other.latches
@@ -75,7 +77,6 @@ class AIGBV:
             lmap=self.lmap + other.lmap,
         )
 
-    @support_inheritance
     def __or__(self, other):
         assert not self.outputs & other.outputs
         assert not self.latches & other.latches
@@ -109,14 +110,16 @@ class AIGBV:
         bmap1 = getattr(self, attr_name)
         assert not set(relabels.values()) & set(bmap1.keys())
         bmap2 = bmap1.relabel(relabels)
-        circ = attr.evolve(self, **{attr_name: bmap2})
 
         # Update AIG to match new interface.
         relabels_aig = fn.merge(*(
             dict(zip(bmap1[k], bmap2[v])) for k, v in relabels.items()
             if k in bmap1
         ))
-        return attr.evolve(circ, aig=circ.aig[kind, relabels_aig])
+
+        return attr.evolve(
+            self, aig=self.aig[kind, relabels_aig], **{attr_name: bmap2}
+        )
 
     def loopback(self, *wirings):
         def wire(circ, wiring):
